@@ -34,6 +34,7 @@ type DeployOptions struct {
 	logger      logger.Logger
 
 	AppFlags            Flags
+	PrevAppFlags        PrevAppFlags
 	FileFlags           cmdtools.FileFlags
 	DiffFlags           cmdtools.DiffFlags
 	ResourceFilterFlags cmdtools.ResourceFilterFlags
@@ -79,6 +80,7 @@ func NewDeployCmd(o *DeployOptions, flagsFactory cmdcore.FlagsFactory) *cobra.Co
 	o.DeployFlags.Set(cmd)
 	o.ResourceTypesFlags.Set(cmd)
 	o.LabelFlags.Set(cmd)
+	o.PrevAppFlags.Set(cmd)
 
 	return cmd
 }
@@ -96,10 +98,10 @@ func (o *DeployOptions) Run() error {
 		return err
 	}
 
-	if o.DeployFlags.PrevApp != "" {
-		err = app.RenamePrevApp(o.DeployFlags.PrevApp, appLabels)
+	if o.PrevAppFlags.PrevAppName != "" {
+		err = app.RenamePrevApp(o.PrevAppFlags.PrevAppName, appLabels, o.DiffFlags.Run)
 	} else {
-		err = app.CreateOrUpdate(appLabels)
+		err = app.CreateOrUpdate(appLabels, o.DiffFlags.Run)
 	}
 
 	if err != nil {
@@ -264,6 +266,15 @@ func (o *DeployOptions) newAndUsedGKs(newGKs []schema.GroupKind, app ctlapp.App)
 		}
 	}
 
+	podGK := schema.GroupKind{
+		Kind: "Pod",
+	}
+
+	// adding Pod in GKs to get existing Pod resources (#460)
+	if _, exists := gksByGK[podGK]; !exists && o.DeployFlags.Logs {
+		uniqGKs = append(uniqGKs, podGK)
+	}
+
 	return uniqGKs, nil
 }
 
